# -*- coding: utf-8 -*-
__version__ = '0.4.3'

__version_text__ = '''giterm version {0} -
    Copyright © 2015-2016 Tim Legrand -
    License BSD 2-Clause License -
    This is free software; see the source for copying conditions. There is NO
    warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE
    '''.format(__version__)
